import dataclasses
from collections.abc import Callable, Coroutine
from typing import Any, Generic, TypeAlias, TypeVar
from pydantic import BaseModel


_T = TypeVar("_T")

ValidatorFunc: TypeAlias = Callable[[Any], Coroutine[Any, Any, None]]


class ErrorSchema(BaseModel, Generic[_T]):
    code: str
    message: str
    detail: str|None = None
    source: _T|None = None


class ValidationError(Generic[_T], Exception):
    messages: list[ErrorSchema[_T]]

    def __init__(
        self,
        messages: list[ErrorSchema[_T]],
        *args: object
    ) -> None:
        super().__init__(*args)
        self.messages = messages


@dataclasses.dataclass
class ValidationContext(Generic[_T]):
    _errors: list[ErrorSchema[_T]]

    def add_error(self, error: ErrorSchema[_T]) -> None:
        self._errors.append(error)

    def extend_nested_error(self, errors: list[ErrorSchema[_T]]) -> None:
        self._errors.extend(errors)

    @property
    def errors(self) -> list[ErrorSchema[_T]]:
        return self._errors



import asyncio
import dataclasses
from pydantic import BaseModel
from business_validator import (
    Validator,
    ValidationError,
    ErrorSchema,
    validate,
)


class CommentDto(BaseModel):
    comment: str
    post_id: int
    owner_id: int


class Source(BaseModel):
    local: str
    position: int


@dataclasses.dataclass
class CommentValidator(Validator[Source]):
    dto_list: list[CommentDto]

    @validate()
    async def test1(self):
        post_ids = range(1, 10)
        for i, dto in enumerate(self.dto_list):
            if dto.post_id not in post_ids:
                self.context.add_error(
                    ErrorSchema(
                        code=12,
                        message="Id doen't not exists",
                        detail=f"Post with id={dto.post_id} not found",
                        source=Source(
                            local="data/post_id",
                            position=i,
                        ),
                    )
                )

    @validate()
    async def test2(self):
        owner_ids = range(1, 10)
        for i, dto in enumerate(self.dto_list):
            if dto.owner_id not in owner_ids:
                self.context.add_error(
                    ErrorSchema(
                        code=12,
                        message="Id doen't not exists",
                        detail=f"User with id={dto.post_id} not found",
                        source=Source(
                            local="data/owner_id",
                            position=i,
                        )
                    )
                )

if __name__ == "__main__":
    try:
        coroutine = CommentValidator(
            [
                CommentDto(comment="comment", owner_id=1, post_id=1),
                CommentDto(comment="comment", owner_id=1, post_id=100),
                CommentDto(comment="comment", owner_id=100, post_id=1),
                CommentDto(comment="comment", owner_id=1234, post_id=1234),
            ]
        ).validate()
        asyncio.run(coroutine)
    except ValidationError as e:
        for message in e.messages:
            print(message)



import asyncio
import dataclasses
from pydantic import BaseModel
from business_validator import (
    Validator,
    ValidationError,
    ErrorSchema,
    validate,
)


class CommentDto(BaseModel):
    comment: str
    post_id: int
    owner_id: int


class Source(BaseModel):
    local: str


@dataclasses.dataclass
class CommentValidator(Validator[Source]):
    dto: CommentDto

    @validate()
    async def test1(self):
        post_ids = list(range(1, 10))

        if self.dto.post_id not in post_ids:
            self.context.add_error(
                ErrorSchema(
                    code=12,
                    message="Id doen't not exists",
                    detail=f"Post with id={self.dto.post_id} not found",
                    source=Source(
                        local="data/post_id",
                    ),
                )
            )

    @validate()
    async def test2(self):
        owner_ids = list(range(1, 10))

        if self.dto.owner_id not in owner_ids:
            self.context.add_error(
                ErrorSchema(
                    code=12,
                    message="Id doen't not exists",
                    detail=f"User with id={self.dto.post_id} not found",
                    source=Source(
                        local="data/owner_id",
                    )
                )
            )


if __name__ == "__main__":
    try:
        asyncio.run(
            CommentValidator(
                CommentDto(
                    comment="comment",
                    owner_id=1234,
                    post_id=222
                )
            ).validate()
        )
    except ValidationError as e:
        print(e.messages)
